Strand

Struct Strand 

pub struct Strand(/* private fields */);
A Strand represents the metadata for the chain of Tixels

A Strand is verified for integrity and authenticity via the CID and public key it contains.

The most common way to obtain a Strand will be from a crate::twine::Twine, a crate::resolver::Resolver, or using Strand::from_tagged_dag_json or Strand::from_block.

A Strand contains an Arc to its underlying data, so it is efficient to clone and pass around.

See also

Implementations

Source

impl Strand

pub fn try_new<C>(container: C) -> Result<Self, VerificationError>

Create a new Strand from its underlying data

It is uncommon to use this directly.

pub fn cid(&self) -> Cid

Get the CID of the Strand

pub fn key(&self) -> PublicKey

Get the public key of the Strand

pub fn radix(&self) -> u8

Get the radix value of the skiplist

pub fn spec_str(&self) -> &str

Get the spec string of the Strand

pub fn version(&self) -> Version

Get the twine version of this record

pub fn subspec(&self) -> Option<Subspec>

Get the subspec of this record if it exists

pub fn details(&self) -> &Ipld

Get the details of the Strand

Details describe the Strand in a human-readable way and can contain any data, usually described by the subspec.

pub fn extract_details<T: DeserializeOwned>( &self, ) -> Result<T, VerificationError>

Deserialize the details as a specific type

pub fn expiry(&self) -> Option<DateTime<Utc>>

Get the expiry date of the Strand

pub fn verify_tixel(&self, tixel: &Tixel) -> Result<(), VerificationError>

Verify a Tixel using this Strand’s public key

pub fn hasher(&self) -> Code

Get the hasher (Code) used to compute the CID