pub struct Strand(/* private fields */);Expand description
A Strand represents the metadata for the chain of Tixels
A Strand is verified for integrity and authenticity via the CID and public key it contains.
The most common way to obtain a Strand will be from a crate::twine::Twine,
a crate::resolver::Resolver, or using Strand::from_tagged_dag_json
or Strand::from_block.
A Strand contains an Arc to its underlying data, so it is efficient to clone and pass around.
See also
Implementations
Sourceimpl Strand
impl Strand
Sourcepub fn try_new<C>(container: C) -> Result<Self, VerificationError>where
C: TryInto<StrandSchemaVersion>,
VerificationError: From<<C as TryInto<StrandSchemaVersion>>::Error>,
pub fn try_new<C>(container: C) -> Result<Self, VerificationError>where
C: TryInto<StrandSchemaVersion>,
VerificationError: From<<C as TryInto<StrandSchemaVersion>>::Error>,
Create a new Strand from its underlying data
It is uncommon to use this directly.
Sourcepub fn details(&self) -> &Ipld
pub fn details(&self) -> &Ipld
Get the details of the Strand
Details describe the Strand in a human-readable way and can contain any data, usually described by the subspec.
Sourcepub fn extract_details<T: DeserializeOwned>(
&self,
) -> Result<T, VerificationError>
pub fn extract_details<T: DeserializeOwned>( &self, ) -> Result<T, VerificationError>
Deserialize the details as a specific type
Sourcepub fn verify_tixel(&self, tixel: &Tixel) -> Result<(), VerificationError>
pub fn verify_tixel(&self, tixel: &Tixel) -> Result<(), VerificationError>
Verify a Tixel using this Strand’s public key